@@ -9,7 +9,12 @@ import os
 import importlib
 import pegtree
 import pegtree.treeconv as treeconv
+from pegtree.terminal import DefaultConsole
 
+bold = DefaultConsole.bold
+color = DefaultConsole.color
+
+'''
 istty = True
 
 
@@ -29,8 +34,6 @@ COLOR = {
 }
 
 
-def color(c, s):
-    return '\033[{}m{}\033[0m'.format(COLOR[c], str(s)) + '' if istty else str(s)
 
 
 def showing(pos, msg):
@@ -39,7 +42,6 @@ def showing(pos, msg):
     else:
         print(pos.showing(msg))
 
-
 def log(type, pos, *msg):
     msg = ' '.join(map(str, msg))
     if type.startswith('err'):
@@ -50,10 +52,10 @@ def log(type, pos, *msg):
         showing(pos, color('Cyan', '[info] ') + str(msg))
     else:
         showing(pos, str(msg))
-
+'''
 
 def version():
-    print(bold('PEGTree Parsing for Python3'))
+    print(DefaultConsole.bold('PEGTree - A PEG Parser Generator with Tree Annotation'))
 
 
 def read_inputs(a):
@@ -81,34 +83,49 @@ def readlines(prompt):
                 break
         return '\n'.join(l)
 
-
 def parse_options(argv):
-    options = {
-        'grammar': ['-g', '--grammar'],
-        'start': ['-s', '--start'],
-        'parser': ['-p', '--parser'],
-        'output': ['-o', '--output'],
-        'verbose': ['--verbose'],
-    }
 
+
+    options = {
+        '-v': ('verbose', True),
+        '-verbose': ('verbose', True),
+        '-g': ('grammar', None),
+        '--grammar': ('grammar', None),
+        '-s': ('start', None),
+        '--start': ('start', None),
+        '-e': ('expression', None),
+        '--expr': ('expression', None),
+        '-f': ('format', None),
+        '--format': ('format', None),
+        '-O0': ('-O', 0),
+        '-O1': ('-O', 1),
+        '-O2': ('-O', 2),
+        '-O2': ('-O', 3),
+    }
     def parse_each(a, d):
-        if a[0].startswith('-'):
-            if len(a) > 1:
-                for key, list in options.items():
-                    for l in list:
-                        if a[0] == l:
-                            d[key] = a[1]
-                            return a[2:]
-            d['inputs'].append(a)
+        first = a[0]
+        if first.startswith('-'):
+            if first in options:
+                key, value = options[first]
+                if value is None:
+                    if len(a) > 1:
+                        d[key] = a[1]
+                        return a[2:]
+                else:
+                    d[key] = value
+                    return a[1:]
+            #d['inputs'].append(a)
             raise CommandUsageError
         else:
             d['inputs'].append(a[0])
             return a[1:]
 
-    d = {'inputs': []}
+    d = {'inputs': [], '-O': 2, 'verbose': False}
     while len(argv) > 0:
         argv = parse_each(argv, d)
-    d['logger'] = log
+    #print('OPTION', d)
+    if d['verbose'] :
+        DefaultConsole.isverbose = True
     return d
 
 
@@ -117,33 +134,39 @@ class CommandUsageError(Exception):
 
 
 def usage():
+    print(bold('PEGTree - A PEG Parser Generator with Tree Annotation'))
     print("Usage: pegtree <command> options inputs")
     print("  -g | --grammar <file>      specify a grammar file")
+    print("  -e | --expr <expression>   specify a parsing expression")
     print("  -s | --start <NAME>        specify a starting rule")
-    print("  -o | --output <file>       specify an output file")
-    print("  -D                         specify an optional value")
+    print("  -f | --format <file>       specify an output format")
     print()
 
     print("Example:")
     print("  pegtree parse -g math.tpeg <inputs>")
     print("  pegtree example -g math.tpeg <inputs>")
     print("  pegtree pasm -g math.tpeg")
+    print("  pegtree update")
     print()
 
     print("The most commonly used pegtree commands are:")
-    print(" parse      run an interactive parser")
-    print(" pasm       generate a parser combinator function")
+    print(" parse      run a generated parser")
     print(" example    test all examples")
+    print(" pasm       generate a pasm combinator function")
+    print(" list       all sample grammars")
     print(" update     update pegtree (via pip)")
 
-
 showingTPEG = False
 
-
 def load_grammar(options, default=None):
     global showingTPEG
+    expr = options.get('expression', None)
+    if expr is not None:
+        grammar = 'A = ' + expr
+        options['urn'] = f'(-e {repr(expr)})'
+        return pegtree.grammar(grammar, **options)
     file = options.get('grammar', default)
-    if file is None:
+    if file is None:        
         print('Enter a TPEG grammar')
         sb = []
         try:
@@ -155,20 +178,21 @@ def load_grammar(options, default=None):
         except:
             pass
         data = '\n'.join(sb)
-        options['basepath'] = '(stdin)'
+        options['urn'] = '(stdin)'
         showingTPEG = False
         return pegtree.grammar(data, **options)
     if file == 'stdin.tpeg':
         data = sys.stdin.read()
-        options['basepath'] = file
+        options['urn'] = file
         return pegtree.grammar(data, **options)
+    options['urn'] = file
     return pegtree.grammar(file, **options)
 
 
 def generator(options):
-    if 'parser' in options:
-        m = importlib.import_module(options['parser'])
-        return m.generate
+    #if 'parser' in options:
+    #    m = importlib.import_module(options['parser'])
+    #    return m.generate
     return pegtree.generate
 
 
@@ -178,30 +202,69 @@ def getstart(peg, options):
     return peg.start()
 
 
-def dump(t):
+def colorTree(t):
     if t.isSyntaxError():
         return t.message(color('Red', 'Syntax Error'))
     else:
-        unconsumed = ''
-        if t.epos_ < len(t.inputs_):
-            unconsumed = ' + ' + color('Purple', t.inputs_[t.epos_:])
+        # unconsumed = ''
+        # if epos < len(t.inputs_):
+        #     unconsumed = ' + ' + color('Purple', t.inputs_[epos:])
         sb = []
         t.strOut(sb, token=lambda x: color('Blue', x),
                  tag=lambda x: color('Cyan', x))
-        return "".join(sb) + unconsumed
+        return "".join(sb)
 
 # parse command
 
+def sample(options, conv=None):
+    files = os.listdir(Path(__file__).parent / 'grammar')
+    files.sort()
+    for file in files:
+        if file.endswith('.tpeg'):
+            print(file)
+
+
+def show(options, conv=None):
+    peg = load_grammar(options)
+    print(peg)
+    if '@@example' in peg:
+        print()
+        prefix = color('Blue', 'example')
+        quote = color("Red", "'''")
+        for testcase in peg['@@example']:
+            name, doc = testcase
+            name = bold(name)
+            doc = doc.getToken()
+            if doc.find('\n') > 0:
+                print(prefix, name, quote)
+                print(doc, quote, sep='\n')
+            else:
+                print(prefix, name, doc)
+
+
+def peg(options):
+    options['isPurePEG'] = True
+    peg = load_grammar(options)
+    print(peg)
+
+
+def optimize(options):
+    from pegtree.optimizer import prepare
+    peg = load_grammar(options)
+    start, refs, rules, memos = prepare(peg)
+    for ref in refs:
+        uname = ref.uname(peg)
+        print(uname, '=', rules[uname])
+    print('memo:', ' '.join(memos))
+
 
 def parse(options, conv=None):
     peg = load_grammar(options)
     parser = generator(options)(peg, **options)
     inputs = options['inputs']
-    tdump = treeconv.treedump(options, dump)
+    tdump = treeconv.treedump(options, colorTree)
     if len(inputs) == 0:  # Interactive Mode
         try:
-            if showingTPEG:
-                print(peg)
             start = getstart(peg, options)
             while True:
                 s = readlines(color('Blue', start) + bold(' <<< '))
@@ -209,7 +272,7 @@ def parse(options, conv=None):
         except (EOFError, KeyboardInterrupt):
             pass
     elif len(inputs) == 1:
-        dump(read_inputs(inputs[0]))
+        colorTree(read_inputs(inputs[0]))
     else:
         for file in options['inputs']:
             st = time.time()
@@ -235,7 +298,7 @@ def example(options):
         fail = doc.inputs_[res.epos_:doc.epos_]
         print(bold(f'parsing {name}'), color(
             'Green', f'{ok}')+color('Red', f'{fail}'), bold('=> '), end='')
-        dump(res)
+        print(colorTree(res))
 
 
 def dumpError(lines, line, s):
@@ -258,35 +321,26 @@ def test(options):
     st = time.time()
     lines = 0
     fail = 0
-    for file in options['inputs']:
-        with open(file) as f:
-            for line in f:
-                lines += 1
-                try:
-                    t = parser(line)
-                    fail += dumpError(lines, line, t)
-                except RecursionError:
-                    print(color('Red', line))
-                    fail += 1
+    try:
+        for file in options['inputs']:
+            with open(file) as f:
+                for line in f:
+                    lines += 1
+                    try:
+                        t = parser(line)
+                        fail += dumpError(lines, line, t)
+                    except RecursionError:
+                        print(color('Red', line))
+                        fail += 1
+    except KeyboardInterrupt:
+        pass
     et = time.time()
     if lines > 0:
         print(f'{fail}/{lines} {fail/lines} {(et - st) * 1000.0} ms')
 
 
-def peg(options):
-    peg = load_grammar(options)
-    print(peg)
-
-
-def pasm0(options):
-    from pegtree.parsec import parsec
-    peg = load_grammar(options)
-    options['optimized'] = 0
-    parsec(peg, **options)
-
-
 def pasm(options):
-    from pegtree.parsec import parsec
+    from pegtree.nezcc import parsec
     peg = load_grammar(options)
     parsec(peg, **options)
 
@@ -302,13 +356,12 @@ def update(options):
 
 def update_beta(options):
     try:
-        # pip3 install -U git+https://github.com/KuramitsuLab/pegpy.git
+        # pip3 install -U git+https://github.com/KuramitsuLab/pegtree.git
         subprocess.check_call(
             ['pip3', 'install', '-U', 'git+https://github.com/KuramitsuLab/pegtree.git'])
     except:
         pass
 
-
 def main(argv=sys.argv):
     names = globals()
     if len(argv) > 1:
@@ -324,5 +377,6 @@ def main(argv=sys.argv):
     usage()
 
 
+
 if __name__ == "__main__":
     main(sys.argv)
